############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import time
import zlib
import queue
import os
from xml.etree import ElementTree
import PyQt5
import indi.indi_xml as indiXML
import astropy.io.fits as pyfits
from baseclasses import checkIP


class INDIClient(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)
    status = PyQt5.QtCore.pyqtSignal(int)
    statusCCD = PyQt5.QtCore.pyqtSignal(bool)
    statusEnvironment = PyQt5.QtCore.pyqtSignal(bool)
    statusDome = PyQt5.QtCore.pyqtSignal(bool)
    receivedImage = PyQt5.QtCore.pyqtSignal(bool)
    processMessage = PyQt5.QtCore.pyqtSignal(object)

    signalDestruct = PyQt5.QtCore.pyqtSignal()

    # INDI device types
    GENERAL_INTERFACE = 0
    TELESCOPE_INTERFACE = (1 << 0)
    CCD_INTERFACE = (1 << 1)
    GUIDER_INTERFACE = (1 << 2)
    FOCUSER_INTERFACE = (1 << 3)
    FILTER_INTERFACE = (1 << 4)
    DOME_INTERFACE = (1 << 5)
    GPS_INTERFACE = (1 << 6)
    WEATHER_INTERFACE = (1 << 7)
    AO_INTERFACE = (1 << 8)
    DUSTCAP_INTERFACE = (1 << 9)
    LIGHTBOX_INTERFACE = (1 << 10)
    DETECTOR_INTERFACE = (1 << 11)
    AUX_INTERFACE = (1 << 15)

    CYCLE = 200
    CONNECTION_TIMEOUT = 2000

    data = {
        'ServerIP': '',
        'ServerPort': 7624,
        'Device': {}
    }

    def __init__(self, app, thread):
        super().__init__()

        self.app = app
        self.thread = thread
        self.isRunning = False
        self.connectCounter = 0
        self.tagFrame = ''
        self.messageString = ''
        self.mutexIPChange = PyQt5.QtCore.QMutex()
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.checkIP = checkIP.CheckIP()
        self.socket = None
        self.newDeviceQueue = queue.Queue()
        self.imagePath = ''
        self.cameraDevice = ''
        self.environmentDevice = ''
        self.domeDevice = ''
        self.telescopeDevice = ''
        self.auxDevice = ''
        self.cycleTimer = None
        # signal slot
        self.app.ui.le_INDIServerIP.editingFinished.connect(self.changedINDIClientConnectionSettings)
        self.app.ui.le_INDIServerPort.editingFinished.connect(self.changedINDIClientConnectionSettings)
        self.app.ui.checkEnableINDI.stateChanged.connect(self.enableDisableINDI)

    def initConfig(self):
        try:
            if 'INDIServerPort' in self.app.config:
                self.app.ui.le_INDIServerPort.setText(self.app.config['INDIServerPort'])
            if 'INDIServerIP' in self.app.config:
                self.app.ui.le_INDIServerIP.setText(self.app.config['INDIServerIP'])
            if 'CheckEnableINDI' in self.app.config:
                self.app.ui.checkEnableINDI.setChecked(self.app.config['CheckEnableINDI'])
            if 'CheckEnableINDIListening' in self.app.config:
                self.app.ui.checkEnableINDIListening.setChecked(self.app.config['CheckEnableINDIListening'])
            if 'CheckEnableINDISolving' in self.app.config:
                self.app.ui.checkEnableINDISolving.setChecked(self.app.config['CheckEnableINDISolving'])
        except Exception as e:
            self.logger.error('item in config.cfg not be initialize, error:{0}'.format(e))
        finally:
            pass
        # setting changes in gui on false, because the set of the config changed them already
        self.changedINDIClientConnectionSettings()
        self.enableDisableINDI()

    def storeConfig(self):
        self.app.config['INDIServerPort'] = self.app.ui.le_INDIServerPort.text()
        self.app.config['INDIServerIP'] = self.app.ui.le_INDIServerIP.text()
        self.app.config['CheckEnableINDI'] = self.app.ui.checkEnableINDI.isChecked()
        self.app.config['CheckEnableINDIListening'] = self.app.ui.checkEnableINDIListening.isChecked()
        self.app.config['CheckEnableINDISolving'] = self.app.ui.checkEnableINDISolving.isChecked()

    def changedINDIClientConnectionSettings(self):
        if self.isRunning:
            self.mutexIPChange.lock()
            self.stop()
            self.data['ServerIP'] = self.app.ui.le_INDIServerIP.text()
            self.data['ServerPort'] = int(self.app.ui.le_INDIServerPort.text())
            self.thread.start()
            self.mutexIPChange.unlock()
        else:
            self.mutexIPChange.lock()
            self.data['ServerIP'] = self.app.ui.le_INDIServerIP.text()
            self.data['ServerPort'] = int(self.app.ui.le_INDIServerPort.text())
            self.mutexIPChange.unlock()
        self.app.messageQueue.put('Setting IP address for INDI to: {0}:{1}\n'.format(self.data['ServerIP'], self.data['ServerPort']))

    def enableDisableINDI(self):
        if self.app.ui.checkEnableINDI.isChecked():
            if not self.isRunning:
                self.thread.start()
        else:
            if self.isRunning:
                self.stop()
            self.status.emit(0)

    def run(self):
        self.logger.info('indi started')
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.socket = PyQt5.QtNetwork.QTcpSocket()
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.LowDelayOption, 1)
        self.socket.setSocketOption(PyQt5.QtNetwork.QAbstractSocket.KeepAliveOption, 1)
        self.socket.hostFound.connect(self.handleHostFound)
        self.socket.connected.connect(self.handleConnected)
        self.socket.stateChanged.connect(self.handleStateChanged)
        self.socket.disconnected.connect(self.handleDisconnect)
        self.socket.readyRead.connect(self.handleReadyRead)
        self.socket.error.connect(self.handleError)
        self.processMessage.connect(self.handleReceived)
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        # if I leave the loop, I close the connection to remote host
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.logger.info('indi client stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
            self.socket.disconnectFromHost()
        self.cycleTimer.stop()
        self.signalDestruct.disconnect(self.destruct)
        self.socket.hostFound.disconnect(self.handleHostFound)
        self.socket.connected.disconnect(self.handleConnected)
        self.socket.stateChanged.disconnect(self.handleStateChanged)
        self.socket.disconnected.disconnect(self.handleDisconnect)
        self.socket.readyRead.disconnect(self.handleReadyRead)
        self.socket.error.disconnect(self.handleError)
        self.socket.abort()

    def doCommand(self):
        self.doReconnect()
        self.handleNewDevice()
        self.app.sharedINDIDataLock.lockForRead()
        if not self.app.INDICommandQueue.empty() and (self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState):
            indiCommand = self.app.INDICommandQueue.get()
            self.sendMessage(indiCommand)
        self.app.sharedINDIDataLock.unlock()

    def doReconnect(self):
        if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.UnconnectedState:
            if self.connectCounter == 0:
                self.app.sharedINDIDataLock.lockForRead()
                self.socket.connectToHost(self.data['ServerIP'], self.data['ServerPort'])
                self.app.sharedINDIDataLock.unlock()
            else:
                # connection build up is ongoing
                pass
            if self.connectCounter * self.CYCLE > self.CONNECTION_TIMEOUT:
                self.socket.abort()
                self.connectCounter = 0
            else:
                self.connectCounter += 1
        else:
            if self.socket.state() != PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
                if self.connectCounter * self.CYCLE > self.CONNECTION_TIMEOUT:
                    self.socket.abort()
                    self.connectCounter = 0
                else:
                    self.connectCounter += 1
            else:
                # connected
                pass

    @PyQt5.QtCore.pyqtSlot()
    def handleHostFound(self):
        self.app.sharedINDIDataLock.lockForRead()
        self.logger.debug('INDI Server found at {}:{}'.format(self.data['ServerIP'], self.data['ServerPort']))
        self.app.sharedINDIDataLock.unlock()

    @PyQt5.QtCore.pyqtSlot()
    def handleConnected(self):
        self.app.sharedINDIDataLock.lockForRead()
        self.logger.info('INDI Server connected at {0}:{1}'.format(self.data['ServerIP'], self.data['ServerPort']))
        self.app.sharedINDIDataLock.unlock()
        # get all informations about existing devices on the choosen indi server
        self.app.INDICommandQueue.put(indiXML.clientGetProperties(indi_attr={'version': '1.7'}))

    def handleNewDevice(self):
        if not self.newDeviceQueue.empty():
            device = self.newDeviceQueue.get()
            # now place the information about accessible devices in the gui and set the connection status
            # and configure the new devices adequately
            # todo: handling of multiple devices of one type and doing the selection
            self.app.sharedINDIDataLock.lockForRead()
            if device in self.data['Device']:
                if 'DRIVER_INFO' in self.data['Device'][device]:
                    if int(self.data['Device'][device]['DRIVER_INFO']['DRIVER_INTERFACE']) & self.CCD_INTERFACE:
                        # make a shortcut for later use and knowing which is a Camera
                        self.cameraDevice = device
                        self.app.INDICommandQueue.put(
                            indiXML.newSwitchVector([indiXML.oneSwitch('On', indi_attr={'name': 'ABORT'})],
                                                    indi_attr={'name': 'CCD_ABORT_EXPOSURE', 'device': self.app.workerINDI.cameraDevice}))
                    elif int(self.data['Device'][device]['DRIVER_INFO']['DRIVER_INTERFACE']) & self.WEATHER_INTERFACE:
                        # make a shortcut for later use
                        self.environmentDevice = device
                    elif int(self.data['Device'][device]['DRIVER_INFO']['DRIVER_INTERFACE']) & self.TELESCOPE_INTERFACE:
                        # make a shortcut for later use
                        self.telescopeDevice = device
                    elif int(self.data['Device'][device]['DRIVER_INFO']['DRIVER_INTERFACE']) & self.DOME_INTERFACE:
                        # make a shortcut for later use
                        self.domeDevice = device
                    elif device == 'SQM':
                        self.auxDevice = device
                    # elif int(self.data['Device'][device]['DRIVER_INFO']['DRIVER_INTERFACE']) == 0:
                        # make a shortcut for later use
                else:
                    # if not ready, put it on the stack again !
                    self.newDeviceQueue.put(device)
            self.app.sharedINDIDataLock.unlock()

    @PyQt5.QtCore.pyqtSlot(PyQt5.QtNetwork.QAbstractSocket.SocketError)
    def handleError(self, socketError):
        if self.socket.error() > 0:
            self.logger.warning('INDI client connection fault, error: {0}'.format(socketError))

    @PyQt5.QtCore.pyqtSlot()
    def handleStateChanged(self):
        self.status.emit(self.socket.state())
        self.logger.debug('INDI client connection has state: {0}'.format(self.socket.state()))

    @PyQt5.QtCore.pyqtSlot()
    def handleDisconnect(self):
        self.logger.info('INDI client connection is disconnected from host')
        self.app.sharedINDIDataLock.lockForWrite()
        self.data['Device'] = {}
        self.app.sharedINDIDataLock.unlock()
        self.cameraDevice = ''
        self.environmentDevice = ''
        self.domeDevice = ''
        self.telescopeDevice = ''
        self.app.INDIStatusQueue.put({'Name': 'Environment', 'value': '---'})
        self.app.INDIStatusQueue.put({'Name': 'CCD', 'value': '---'})
        self.app.INDIStatusQueue.put({'Name': 'Dome', 'value': '---'})
        self.app.INDIStatusQueue.put({'Name': 'Telescope', 'value': '---'})
        self.app.INDIStatusQueue.put({'Name': 'Aux', 'value': '---'})

    @PyQt5.QtCore.pyqtSlot(object)
    def handleReceived(self, message):
        # central dispatcher for data coming from INDI devices. I makes the whole status and data evaluation and fits the
        # data to mountwizzard3
        if 'device' not in message.attr:
            return
        device = message.attr['device']
        # receiving all definitions for vectors in indi and building them up in self.data['Device']
        if isinstance(message, indiXML.DefBLOBVector):
            self.app.sharedINDIDataLock.lockForWrite()
            if device not in self.data['Device']:
                self.data['Device'][device] = {}
            if device in self.data['Device']:
                if 'name' in message.attr:
                    defVector = message.attr['name']
                    if defVector not in self.data['Device'][device]:
                        self.data['Device'][device][defVector] = {}
                    for elt in message.elt_list:
                        self.data['Device'][device][defVector][elt.attr['name']] = ''
            self.app.sharedINDIDataLock.unlock()

        elif isinstance(message, indiXML.SetBLOBVector):
            self.app.sharedINDIDataLock.lockForRead()
            if device in self.data['Device']:
                if int(self.data['Device'][device]['DRIVER_INFO']['DRIVER_INTERFACE']) & self.CCD_INTERFACE:
                    name = message.attr['name']
                    # ccd1 is the main camera in INDI
                    if name == 'CCD1':
                        # format tells me raw or compressed format
                        if 'format' in message.getElt(0).attr:
                            try:
                                if self.imagePath != '':
                                    if message.getElt(0).attr['format'] == '.fits':
                                        imageHDU = pyfits.HDUList.fromstring(message.getElt(0).getValue())
                                        imageHDU.writeto(self.imagePath, overwrite=True)
                                        self.logger.debug('Image BLOB is in raw fits format')
                                    else:
                                        imageHDU = pyfits.HDUList.fromstring(zlib.decompress(message.getElt(0).getValue()))
                                        imageHDU.writeto(self.imagePath, overwrite=True)
                                        self.logger.debug('Image BLOB is compressed fits format')
                                    self.receivedImage.emit(True)
                                else:
                                    # here are some functions to listen to image transfer on ethernet
                                    if self.app.ui.checkEnableINDIListening.isChecked():
                                        # received an image without asking for it. just listening
                                        path = os.getcwd() + '/images/listen.fit'
                                        if message.getElt(0).attr['format'] == '.fits':
                                            imageHDU = pyfits.HDUList.fromstring(message.getElt(0).getValue())
                                            imageHDU.writeto(path, overwrite=True)
                                            self.logger.debug('Image while listening is received in raw fits format')
                                        else:
                                            imageHDU = pyfits.HDUList.fromstring(zlib.decompress(message.getElt(0).getValue()))
                                            imageHDU.writeto(path, overwrite=True)
                                            self.logger.debug('Image while listening is received in compressed fits format')
                                        self.app.imageWindow.signalShowFitsImage.emit(path)
                                        # if there is a hint, we could solve it as well automatically
                                        if self.app.ui.checkEnableINDISolving.isChecked():
                                            self.app.imageWindow.signalSolveFitsImage.emit(path)
                                    else:
                                        # do nothing
                                        pass
                            except Exception as e:
                                self.receivedImage.emit(False)
                                self.logger.debug('Could not receive Image, error:{0}'.format(e))
                            finally:
                                pass
                        else:
                            self.logger.debug('Could not find format in message from device: {0}'.format(device))
                    else:
                        self.logger.debug('Got BLOB from device: {0}, name: {1}'.format(device, name))
                else:
                    self.logger.debug('Got unexpected BLOB from device: {0}'.format(device))
            else:
                self.logger.debug('Did not find device: {0} in device list'.format(device))
            self.app.sharedINDIDataLock.unlock()

        # deleting properties from devices
        elif isinstance(message, indiXML.DelProperty):
            self.app.sharedINDIDataLock.lockForWrite()
            if device in self.data['Device']:
                if 'name' in message.attr:
                    delVector = message.attr['name']
                    if delVector in self.data['Device'][device]:
                        del self.data['Device'][device][delVector]
            self.app.sharedINDIDataLock.unlock()

        # receiving changes from vectors and updating them ins self.data['Device]
        elif isinstance(message, indiXML.SetSwitchVector) or \
                isinstance(message, indiXML.SetTextVector) or \
                isinstance(message, indiXML.SetLightVector) or \
                isinstance(message, indiXML.SetNumberVector):
            self.app.sharedINDIDataLock.lockForWrite()
            if device in self.data['Device']:
                if 'name' in message.attr:
                    setVector = message.attr['name']
                    if setVector not in self.data['Device'][device]:
                        self.data['Device'][device][setVector] = {}
                        self.logger.warning('SetVector before DefVector in INDI protocol, device: {0}, vector: {1}'.format(device, setVector))
                    if 'state' in message.attr:
                        self.data['Device'][device][setVector]['state'] = message.attr['state']
                    if 'timeout' in message.attr:
                        self.data['Device'][device][setVector]['timeout'] = message.attr['timeout']
                    for elt in message.elt_list:
                        self.data['Device'][device][setVector][elt.attr['name']] = elt.getValue()
            self.app.sharedINDIDataLock.unlock()

        # receiving all definitions for vectors in indi and building them up in self.data['Device']
        elif isinstance(message, indiXML.DefSwitchVector) or \
                isinstance(message, indiXML.DefTextVector) or \
                isinstance(message, indiXML.DefLightVector) or \
                isinstance(message, indiXML.DefNumberVector):
            self.app.sharedINDIDataLock.lockForWrite()
            if device not in self.data['Device']:
                # new device !
                self.data['Device'][device] = {}
                self.newDeviceQueue.put(device)
            if device in self.data['Device']:
                if 'name' in message.attr:
                    defVector = message.attr['name']
                    if defVector not in self.data['Device'][device]:
                        self.data['Device'][device][defVector] = {}
                    if 'state' in message.attr:
                        self.data['Device'][device][defVector]['state'] = message.attr['state']
                    if 'perm' in message.attr:
                        self.data['Device'][device][defVector]['perm'] = message.attr['perm']
                    if 'timeout' in message.attr:
                        self.data['Device'][device][defVector]['timeout'] = message.attr['timeout']
                    for elt in message.elt_list:
                        self.data['Device'][device][defVector][elt.attr['name']] = elt.getValue()
            self.app.sharedINDIDataLock.unlock()

        self.app.sharedINDIDataLock.lockForRead()
        if device in self.data['Device']:
            if 'DRIVER_INFO' in self.data['Device'][device]:
                if int(self.data['Device'][device]['DRIVER_INFO']['DRIVER_INTERFACE']) & self.CCD_INTERFACE:
                    self.app.INDIStatusQueue.put({'Name': 'CCD', 'value': device})
                elif int(self.data['Device'][device]['DRIVER_INFO']['DRIVER_INTERFACE']) & self.WEATHER_INTERFACE:
                    self.app.INDIStatusQueue.put({'Name': 'Environment', 'value': device})
                elif int(self.data['Device'][device]['DRIVER_INFO']['DRIVER_INTERFACE']) & self.DOME_INTERFACE:
                    self.app.INDIStatusQueue.put({'Name': 'Dome', 'value': device})
                elif int(self.data['Device'][device]['DRIVER_INFO']['DRIVER_INTERFACE']) & self.TELESCOPE_INTERFACE:
                    self.app.INDIStatusQueue.put({'Name': 'Telescope', 'value': device})
                elif int(self.data['Device'][device]['DRIVER_INFO']['DRIVER_INTERFACE']) & self.AUX_INTERFACE:
                    self.app.INDIStatusQueue.put({'Name': 'Aux', 'value': device})
        self.app.sharedINDIDataLock.unlock()

    @PyQt5.QtCore.pyqtSlot()
    def handleReadyRead(self):
        # Add starting tag if this is new message.
        # we have to check and keep the first XML tag
        if len(self.messageString) == 0:
            self.messageString = "<data>"
        # Get message from socket.
        while self.socket.bytesAvailable() and self.isRunning:
            # print(self.socket.bytesAvailable(), len(self.messageString))
            try:
                self.messageString += self.socket.read(100000).decode()
            except Exception as e:
                self.logger.error('Cannot decode, error:{0}'.format(e))
            finally:
                pass
        # Add closing tag.
        self.messageString += "</data>"
        # Try and parse the message !
        try:
            messages = ElementTree.fromstring(self.messageString)
            self.messageString = ""
            for message in messages:
                xmlMessage = indiXML.parseETree(message)
                self.processMessage.emit(xmlMessage)
        # Message is incomplete, remove </data> and try again with more data
        except ElementTree.ParseError:
            self.messageString = self.messageString[:-7]
        finally:
            pass

    @PyQt5.QtCore.pyqtSlot()
    def handleReadyReadNew(self):
        # Get message from socket.
        while self.socket.bytesAvailable() and self.isRunning:
            # decode message and catch error
            try:
                self.messageString += self.socket.read(10000).decode()
            except Exception as e:
                self.logger.error('INDI message raw decode, error:{0}'.format(e))
            finally:
                pass
        # get first tag frame
        if self.tagFrame == '':
            self.tagFrame = self.messageString[1:50].split(' ')[0]
            print('start: ', self.tagFrame, len(self.messageString))
        # checking if closing first tag is there
        if self.messageString.endswith('</' + self.tagFrame + '>') or self.messageString.endswith('</' + self.tagFrame + '>\n'):
            # last part of message, process now
            messageComplete = True
            print('finished: ' + self.tagFrame)
        else:
            # otherwise continue
            messageComplete = False
            print(len(self.messageString), self.messageString[len(self.messageString)-30:], ord(self.messageString[-2]), ord(self.messageString[-1]))

        if messageComplete:
            # adding frame tags for INDI
            self.messageString = "<data>" + self.messageString + "</data>"
            try:
                # decode INDI XML elements
                messages = ElementTree.fromstring(self.messageString)
                # clear message
                for message in messages:
                    # print('message: ', message)
                    xmlMessage = indiXML.parseETree(message)
                    self.processMessage.emit(xmlMessage)
            except ElementTree.ParseError:
                print('error parse')
                self.logger.error('INDI XML message parse error')
            finally:
                print('reset to next round')
                self.messageString = ''
                self.tagFrame = ''
        else:
            # move on
            pass

    def sendMessage(self, indiCommand):
        if self.socket.state() == PyQt5.QtNetwork.QAbstractSocket.ConnectedState:
            self.socket.write(indiCommand.toXML() + b'\n')
            self.socket.flush()
        else:
            self.logger.warning('Socket not connected')



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import time
import PyQt5
import queue
import requests
from baseclasses import checkIP


class Relays(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    CYCLE = 200
    CYCLE_STATUS = 500
    signalDestruct = PyQt5.QtCore.pyqtSignal()

    def __init__(self, app, thread):
        super().__init__()
        self.isRunning = False
        self.app = app
        self.thread = thread
        self.statusTimer = None
        self.cycleTimer = None
        self.relayCommandQueue = queue.Queue()
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.mutexIPChange = PyQt5.QtCore.QMutex()

        self.stat = [False, False, False, False, False, False, False, False, False]
        self.username = ''
        self.password = ''
        self.relayIP = ''
        self.connected = False
        self.checkIP = checkIP.CheckIP()
        self.app.ui.btn_relay1.clicked.connect(lambda: self.relayCommandQueue.put(1))
        self.app.ui.btn_relay2.clicked.connect(lambda: self.relayCommandQueue.put(2))
        self.app.ui.btn_relay3.clicked.connect(lambda: self.relayCommandQueue.put(3))
        self.app.ui.btn_relay4.clicked.connect(lambda: self.relayCommandQueue.put(4))
        self.app.ui.btn_relay5.clicked.connect(lambda: self.relayCommandQueue.put(5))
        self.app.ui.btn_relay6.clicked.connect(lambda: self.relayCommandQueue.put(6))
        self.app.ui.btn_relay7.clicked.connect(lambda: self.relayCommandQueue.put(7))
        self.app.ui.btn_relay8.clicked.connect(lambda: self.relayCommandQueue.put(8))
        # signal slot
        self.app.ui.relay1Text.textEdited.connect(lambda: self.app.ui.btn_relay1.setText(self.app.ui.relay1Text.text()))
        self.app.ui.relay2Text.textEdited.connect(lambda: self.app.ui.btn_relay2.setText(self.app.ui.relay2Text.text()))
        self.app.ui.relay3Text.textEdited.connect(lambda: self.app.ui.btn_relay3.setText(self.app.ui.relay3Text.text()))
        self.app.ui.relay4Text.textEdited.connect(lambda: self.app.ui.btn_relay4.setText(self.app.ui.relay4Text.text()))
        self.app.ui.relay5Text.textEdited.connect(lambda: self.app.ui.btn_relay5.setText(self.app.ui.relay5Text.text()))
        self.app.ui.relay6Text.textEdited.connect(lambda: self.app.ui.btn_relay6.setText(self.app.ui.relay6Text.text()))
        self.app.ui.relay7Text.textEdited.connect(lambda: self.app.ui.btn_relay7.setText(self.app.ui.relay7Text.text()))
        self.app.ui.relay8Text.textEdited.connect(lambda: self.app.ui.btn_relay8.setText(self.app.ui.relay8Text.text()))
        self.app.ui.le_relayIP.editingFinished.connect(self.changedRelayClientConnectionSettings)
        self.app.ui.checkEnableRelay.stateChanged.connect(self.enableDisableRelay)

    def initConfig(self):
        view1 = PyQt5.QtWidgets.QListView()
        self.app.ui.relay1Function.setView(view1)
        self.app.ui.relay1Function.addItem('Switch - Toggle')
        self.app.ui.relay1Function.addItem('Pulse 1 sec')
        view2 = PyQt5.QtWidgets.QListView()
        self.app.ui.relay2Function.setView(view2)
        self.app.ui.relay2Function.addItem('Switch - Toggle')
        self.app.ui.relay2Function.addItem('Pulse 1 sec')
        view3 = PyQt5.QtWidgets.QListView()
        self.app.ui.relay3Function.setView(view3)
        self.app.ui.relay3Function.addItem('Switch - Toggle')
        self.app.ui.relay3Function.addItem('Pulse 1 sec')
        view4 = PyQt5.QtWidgets.QListView()
        self.app.ui.relay4Function.setView(view4)
        self.app.ui.relay4Function.addItem('Switch - Toggle')
        self.app.ui.relay4Function.addItem('Pulse 1 sec')
        view5 = PyQt5.QtWidgets.QListView()
        self.app.ui.relay5Function.setView(view5)
        self.app.ui.relay5Function.addItem('Switch - Toggle')
        self.app.ui.relay5Function.addItem('Pulse 1 sec')
        view6 = PyQt5.QtWidgets.QListView()
        self.app.ui.relay6Function.setView(view6)
        self.app.ui.relay6Function.addItem('Switch - Toggle')
        self.app.ui.relay6Function.addItem('Pulse 1 sec')
        view7 = PyQt5.QtWidgets.QListView()
        self.app.ui.relay7Function.setView(view7)
        self.app.ui.relay7Function.addItem('Switch - Toggle')
        self.app.ui.relay7Function.addItem('Pulse 1 sec')
        view8 = PyQt5.QtWidgets.QListView()
        self.app.ui.relay8Function.setView(view8)
        self.app.ui.relay8Function.addItem('Switch - Toggle')
        self.app.ui.relay8Function.addItem('Pulse 1 sec')
        try:
            if 'Relay1Function' in self.app.config:
                self.app.ui.relay1Function.setCurrentIndex(self.app.config['Relay1Function'])
            if 'Relay2Function' in self.app.config:
                self.app.ui.relay2Function.setCurrentIndex(self.app.config['Relay2Function'])
            if 'Relay3Function' in self.app.config:
                self.app.ui.relay3Function.setCurrentIndex(self.app.config['Relay3Function'])
            if 'Relay4Function' in self.app.config:
                self.app.ui.relay4Function.setCurrentIndex(self.app.config['Relay4Function'])
            if 'Relay5Function' in self.app.config:
                self.app.ui.relay5Function.setCurrentIndex(self.app.config['Relay5Function'])
            if 'Relay6Function' in self.app.config:
                self.app.ui.relay6Function.setCurrentIndex(self.app.config['Relay6Function'])
            if 'Relay7Function' in self.app.config:
                self.app.ui.relay7Function.setCurrentIndex(self.app.config['Relay7Function'])
            if 'Relay8Function' in self.app.config:
                self.app.ui.relay8Function.setCurrentIndex(self.app.config['Relay8Function'])
            if 'Relay1Text' in self.app.config:
                self.app.ui.relay1Text.setText(self.app.config['Relay1Text'])
                self.app.ui.btn_relay1.setText(self.app.config['Relay1Text'])
            if 'Relay2Text' in self.app.config:
                self.app.ui.relay2Text.setText(self.app.config['Relay2Text'])
                self.app.ui.btn_relay2.setText(self.app.config['Relay2Text'])
            if 'Relay3Text' in self.app.config:
                self.app.ui.relay3Text.setText(self.app.config['Relay3Text'])
                self.app.ui.btn_relay3.setText(self.app.config['Relay3Text'])
            if 'Relay4Text' in self.app.config:
                self.app.ui.relay4Text.setText(self.app.config['Relay4Text'])
                self.app.ui.btn_relay4.setText(self.app.config['Relay4Text'])
            if 'Relay5Text' in self.app.config:
                self.app.ui.relay5Text.setText(self.app.config['Relay5Text'])
                self.app.ui.btn_relay5.setText(self.app.config['Relay5Text'])
            if 'Relay6Text' in self.app.config:
                self.app.ui.relay6Text.setText(self.app.config['Relay6Text'])
                self.app.ui.btn_relay6.setText(self.app.config['Relay6Text'])
            if 'Relay7Text' in self.app.config:
                self.app.ui.relay7Text.setText(self.app.config['Relay7Text'])
                self.app.ui.btn_relay7.setText(self.app.config['Relay7Text'])
            if 'Relay8Text' in self.app.config:
                self.app.ui.relay8Text.setText(self.app.config['Relay8Text'])
                self.app.ui.btn_relay8.setText(self.app.config['Relay8Text'])
            if 'RelayIP' in self.app.config:
                self.app.ui.le_relayIP.setText(self.app.config['RelayIP'])
                self.relayIP = self.app.config['RelayIP']
            if 'RelayUsername' in self.app.config:
                self.app.ui.le_relayUsername.setText(self.app.config['RelayUsername'])
            if 'RelayPassword' in self.app.config:
                self.app.ui.le_relayPassword.setText(self.app.config['RelayPassword'])
            if 'CheckEnableRelay' in self.app.config:
                self.app.ui.checkEnableRelay.setChecked(self.app.config['CheckEnableRelay'])
        except Exception as e:
            self.logger.error('Item in config.cfg for relay could not be initialized, error:{0}'.format(e))
        finally:
            pass
        self.changedRelayClientConnectionSettings()
        self.enableDisableRelay()

    def storeConfig(self):
        self.app.config['RelayIP'] = self.app.ui.le_relayIP.text()
        self.app.config['Relay1Function'] = self.app.ui.relay1Function.currentIndex()
        self.app.config['Relay2Function'] = self.app.ui.relay2Function.currentIndex()
        self.app.config['Relay3Function'] = self.app.ui.relay3Function.currentIndex()
        self.app.config['Relay4Function'] = self.app.ui.relay4Function.currentIndex()
        self.app.config['Relay5Function'] = self.app.ui.relay5Function.currentIndex()
        self.app.config['Relay6Function'] = self.app.ui.relay6Function.currentIndex()
        self.app.config['Relay7Function'] = self.app.ui.relay7Function.currentIndex()
        self.app.config['Relay8Function'] = self.app.ui.relay8Function.currentIndex()
        self.app.config['Relay1Text'] = self.app.ui.relay1Text.text()
        self.app.config['Relay2Text'] = self.app.ui.relay2Text.text()
        self.app.config['Relay3Text'] = self.app.ui.relay3Text.text()
        self.app.config['Relay4Text'] = self.app.ui.relay4Text.text()
        self.app.config['Relay5Text'] = self.app.ui.relay5Text.text()
        self.app.config['Relay6Text'] = self.app.ui.relay6Text.text()
        self.app.config['Relay7Text'] = self.app.ui.relay7Text.text()
        self.app.config['Relay8Text'] = self.app.ui.relay8Text.text()
        self.app.config['RelayUsername'] = self.app.ui.le_relayUsername.text()
        self.app.config['RelayPassword'] = self.app.ui.le_relayPassword.text()
        self.app.config['CheckEnableRelay'] = self.app.ui.checkEnableRelay.isChecked()

    def changedRelayClientConnectionSettings(self):
        if self.isRunning:
            self.mutexIPChange.lock()
            self.stop()
            self.relayIP = self.app.ui.le_relayIP.text()
            self.thread.start()
            self.mutexIPChange.unlock()
        else:
            self.mutexIPChange.lock()
            self.relayIP = self.app.ui.le_relayIP.text()
            self.mutexIPChange.unlock()
        self.app.messageQueue.put('Setting IP address for relay to: {0}\n'.format(self.relayIP))

    def enableDisableRelay(self):
        # get relay tab index:
        index = self.app.ui.mainTabWidget.indexOf(self.app.ui.mainTabWidget.findChild(PyQt5.QtWidgets.QWidget, 'Relay'))
        if self.app.ui.checkEnableRelay.isChecked():
            if not self.isRunning:
                self.thread.start()
            self.app.ui.mainTabWidget.setTabEnabled(index, True)
            self.app.messageQueue.put('Relay enabled\n')
        else:
            self.connected = False
            if self.isRunning:
                self.stop()
            self.app.ui.mainTabWidget.setTabEnabled(index, False)
            self.app.messageQueue.put('Relay disabled\n')
            self.logger.info('Relay is disabled')
        self.app.ui.mainTabWidget.style().unpolish(self.app.ui.mainTabWidget)
        self.app.ui.mainTabWidget.style().polish(self.app.ui.mainTabWidget)

    def run(self):
        self.logger.info('relay started')
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        # timers
        self.statusTimer = PyQt5.QtCore.QTimer(self)
        self.statusTimer.setSingleShot(False)
        self.statusTimer.timeout.connect(self.getStatus)
        self.statusTimer.start(self.CYCLE_STATUS)
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        # when the worker thread finished, it emit the finished signal to the parent to clean up
        self.logger.info('relay stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        self.cycleTimer.stop()
        self.statusTimer.stop()
        self.signalDestruct.disconnect(self.destruct)

    def doCommand(self):
        if not self.relayCommandQueue.empty():
            value = self.relayCommandQueue.get()
            self.runRelay(value)

    @PyQt5.QtCore.pyqtSlot()
    def getStatus(self):
        if self.checkIP.checkIPAvailable(self.relayIP, 80):
            try:
                result = self.geturl('http://' + self.relayIP + '/status.xml')
                self.setStatusGUI(result.content.decode())
            except Exception as e:
                self.logger.error('Status error {0}'.format(e))
            finally:
                pass

    def setStatusGUI(self, response):
        lines = response.splitlines()
        if lines[0] == '<response>':
            self.stat[1] = (lines[2][8] == '1')
            self.stat[2] = (lines[3][8] == '1')
            self.stat[3] = (lines[4][8] == '1')
            self.stat[4] = (lines[5][8] == '1')
            self.stat[5] = (lines[6][8] == '1')
            self.stat[6] = (lines[7][8] == '1')
            self.stat[7] = (lines[8][8] == '1')
            self.stat[8] = (lines[9][8] == '1')
            self.logger.debug('status: {0}'.format(self.stat))
        if self.stat[1]:
            self.app.ui.btn_relay1.setProperty('running', True)
            self.app.ui.btn_relay1.style().unpolish(self.app.ui.btn_relay1)
            self.app.ui.btn_relay1.style().polish(self.app.ui.btn_relay1)
        else:
            self.app.ui.btn_relay1.setProperty('running', False)
            self.app.ui.btn_relay1.style().unpolish(self.app.ui.btn_relay1)
            self.app.ui.btn_relay1.style().polish(self.app.ui.btn_relay1)
        if self.stat[2]:
            self.app.ui.btn_relay2.setProperty('running', True)
            self.app.ui.btn_relay2.style().unpolish(self.app.ui.btn_relay2)
            self.app.ui.btn_relay2.style().polish(self.app.ui.btn_relay2)
        else:
            self.app.ui.btn_relay2.setProperty('running', False)
            self.app.ui.btn_relay2.style().unpolish(self.app.ui.btn_relay2)
            self.app.ui.btn_relay2.style().polish(self.app.ui.btn_relay2)
        if self.stat[3]:
            self.app.ui.btn_relay3.setProperty('running', True)
            self.app.ui.btn_relay3.style().unpolish(self.app.ui.btn_relay3)
            self.app.ui.btn_relay3.style().polish(self.app.ui.btn_relay3)
        else:
            self.app.ui.btn_relay3.setProperty('running', False)
            self.app.ui.btn_relay3.style().unpolish(self.app.ui.btn_relay3)
            self.app.ui.btn_relay3.style().polish(self.app.ui.btn_relay3)
        if self.stat[4]:
            self.app.ui.btn_relay4.setProperty('running', True)
            self.app.ui.btn_relay4.style().unpolish(self.app.ui.btn_relay4)
            self.app.ui.btn_relay4.style().polish(self.app.ui.btn_relay4)
        else:
            self.app.ui.btn_relay4.setProperty('running', False)
            self.app.ui.btn_relay4.style().unpolish(self.app.ui.btn_relay4)
            self.app.ui.btn_relay4.style().polish(self.app.ui.btn_relay4)
        if self.stat[5]:
            self.app.ui.btn_relay5.setProperty('running', True)
            self.app.ui.btn_relay5.style().unpolish(self.app.ui.btn_relay5)
            self.app.ui.btn_relay5.style().polish(self.app.ui.btn_relay5)
        else:
            self.app.ui.btn_relay5.setProperty('running', False)
            self.app.ui.btn_relay5.style().unpolish(self.app.ui.btn_relay5)
            self.app.ui.btn_relay5.style().polish(self.app.ui.btn_relay5)
        if self.stat[6]:
            self.app.ui.btn_relay6.setProperty('running', True)
            self.app.ui.btn_relay6.style().unpolish(self.app.ui.btn_relay6)
            self.app.ui.btn_relay6.style().polish(self.app.ui.btn_relay6)
        else:
            self.app.ui.btn_relay6.setProperty('running', False)
            self.app.ui.btn_relay6.style().unpolish(self.app.ui.btn_relay6)
            self.app.ui.btn_relay6.style().polish(self.app.ui.btn_relay6)
        if self.stat[7]:
            self.app.ui.btn_relay7.setProperty('running', True)
            self.app.ui.btn_relay7.style().unpolish(self.app.ui.btn_relay7)
            self.app.ui.btn_relay7.style().polish(self.app.ui.btn_relay7)
        else:
            self.app.ui.btn_relay7.setProperty('running', False)
            self.app.ui.btn_relay7.style().unpolish(self.app.ui.btn_relay7)
            self.app.ui.btn_relay7.style().polish(self.app.ui.btn_relay7)
        if self.stat[8]:
            self.app.ui.btn_relay8.setProperty('running', True)
            self.app.ui.btn_relay8.style().unpolish(self.app.ui.btn_relay8)
            self.app.ui.btn_relay8.style().polish(self.app.ui.btn_relay8)
        else:
            self.app.ui.btn_relay8.setProperty('running', False)
            self.app.ui.btn_relay8.style().unpolish(self.app.ui.btn_relay8)
            self.app.ui.btn_relay8.style().polish(self.app.ui.btn_relay8)

    def geturl(self, url):
        result = requests.get(url, auth=requests.auth.HTTPBasicAuth(self.app.ui.le_relayUsername.text(), self.app.ui.le_relayPassword.text()))
        return result

    def runRelay(self, relayNumber):
        if self.checkIP.checkIPAvailable(self.relayIP, 80):
            if relayNumber == 1:
                if self.app.ui.relay1Function.currentIndex() == 0:
                    self.switch(relayNumber)
                else:
                    self.pulse(relayNumber)
            if relayNumber == 2:
                if self.app.ui.relay2Function.currentIndex() == 0:
                    self.switch(relayNumber)
                else:
                    self.pulse(relayNumber)
            if relayNumber == 3:
                if self.app.ui.relay3Function.currentIndex() == 0:
                    self.switch(relayNumber)
                else:
                    self.pulse(relayNumber)
            if relayNumber == 4:
                if self.app.ui.relay4Function.currentIndex() == 0:
                    self.switch(relayNumber)
                else:
                    self.pulse(relayNumber)
            if relayNumber == 5:
                if self.app.ui.relay5Function.currentIndex() == 0:
                    self.switch(relayNumber)
                else:
                    self.pulse(relayNumber)
            if relayNumber == 6:
                if self.app.ui.relay6Function.currentIndex() == 0:
                    self.switch(relayNumber)
                else:
                    self.pulse(relayNumber)
            if relayNumber == 7:
                if self.app.ui.relay7Function.currentIndex() == 0:
                    self.switch(relayNumber)
                else:
                    self.pulse(relayNumber)
            if relayNumber == 8:
                if self.app.ui.relay8Function.currentIndex() == 0:
                    self.switch(relayNumber)
                else:
                    self.pulse(relayNumber)

    def pulse(self, relayNumber):
        try:
            self.geturl('http://' + self.relayIP + '/FF0{0:1d}01'.format(relayNumber))
            time.sleep(1)
            self.geturl('http://' + self.relayIP + '/FF0{0:1d}00'.format(relayNumber))
        except Exception as e:
            self.logger.error('Relay:{0}, error:{1}'.format(relayNumber, e))
        finally:
            pass

    def switch(self, relayNumber):
        try:
            self.geturl('http://' + self.relayIP + '/relays.cgi?relay={0:1d}'.format(relayNumber))
        except Exception as e:
            self.logger.error('Relay:{0}, error:{1}'.format(relayNumber, e))
        finally:
            pass



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import time
import PyQt5
import platform
from win32com.client.dynamic import Dispatch
import pythoncom


class MaximDL:
    logger = logging.getLogger(__name__)
    TIMEOUT_STOP = 5

    def __init__(self, main, app, data):
        self.main = main
        self.app = app
        self.data = data
        self.cancel = False
        self.mutexCancel = PyQt5.QtCore.QMutex()

        self.application = dict()
        self.application['Available'] = False
        self.application['Name'] = ''
        self.application['InstallPath'] = ''
        self.application['Status'] = ''
        self.application['Runtime'] = 'MaxIm_DL.exe'
        self.maximCamera = None
        self.maximApplication = None

        if platform.system() == 'Windows':
            # sgpro only supported on local machine
            self.application['Available'], self.application['Name'], self.application['InstallPath'] = self.app.checkRegistrationKeys('MaxIm DL')
            if self.application['Available']:
                self.app.messageQueue.put('Found Imaging: {0}\n'.format(self.application['Name']))
                self.logger.info('Name: {0}, Path: {1}'.format(self.application['Name'], self.application['InstallPath']))
            else:
                self.logger.info('Application MaximDL not found on computer')

    def start(self):
        pythoncom.CoInitialize()
        try:
            self.maximCamera = Dispatch('MaxIm.CCDCamera')
            self.maximApplication = Dispatch('MaxIm.Application')
            self.maximCamera.LinkEnabled = True
            # test if link established
            if self.maximCamera.LinkEnabled:
                self.data['CONNECTION']['CONNECT'] = 'On'
                self.logger.info('Maxim started')
                self.application['Status'] = 'OK'
            else:
                self.logger.info('Maxim could not be started, error:{0}'.format(e))
                self.application['Status'] = 'ERROR'
        except Exception as e:
            self.logger.info('Maxim could not be started, error:{0}'.format(e))
            self.application['Status'] = 'ERROR'
        finally:
            pass

    def stop(self):
        try:
            if self.maximCamera:
                self.data['CONNECTION']['CONNECT'] = 'Off'
                self.maximCamera.LinkEnabled = False
                timeStart = time.time()
                while self.maximCamera.LinkEnabled and (time.time() - timeStart) < self.TIMEOUT_STOP:
                    time.sleep(0.1)
                self.maximCamera = None
                self.maximApplication = None
        except Exception as e:
            self.logger.error('Could not stop maxim, error: {0}'.format(e))
            self.application['Status'] = 'ERROR'
        finally:
            self.data['CONNECTION']['CONNECT'] = 'Off'
            self.maximCamera = None
            self.maximApplication = None
            pythoncom.CoUninitialize()

    def getStatus(self):
        if self.maximCamera:
            try:
                if self.maximCamera.LinkEnabled:
                    status = self.maximCamera.CameraStatus
                    if status in [0, 1]:
                        self.data['CONNECTION']['CONNECT'] = 'Off'
                        self.application['Status'] = 'OK'
                    elif status in [2, 3, 5]:
                        self.data['CONNECTION']['CONNECT'] = 'On'
                        self.application['Status'] = 'OK'
                    else:
                        self.logger.error('Unknown camera status: {0}'.format(status))
                        self.application['Status'] = 'ERROR'
                else:
                    self.maximCamera.LinkEnabled = True
            except Exception as e:
                self.stop()
            finally:
                pass
        else:
            self.application['Status'] = 'ERROR'
            self.data['CONNECTION']['CONNECT'] = 'Off'

    def getCameraProps(self):
        if self.maximCamera and self.data['CONNECTION']['CONNECT'] == 'On':
            self.data['Gain'] = ['High']
            self.data['CCD_INFO'] = {}
            self.data['CCD_INFO']['CCD_MAX_X'] = self.maximCamera.CameraXSize
            self.data['CCD_INFO']['CCD_MAX_Y'] = self.maximCamera.CameraYSize
        else:
            self.application['Status'] = 'ERROR'
            self.data['CONNECTION']['CONNECT'] = 'Off'

    def getImage(self, imageParams):
        self.data['Imaging'] = True
        self.mutexCancel.lock()
        self.cancel = False
        self.mutexCancel.unlock()

        # waiting for start integrating
        self.main.cameraStatusText.emit('START')
        try:
            self.maximCamera.BinX = int(imageParams['Binning'])
            self.maximCamera.BinY = int(imageParams['Binning'])
            self.maximCamera.NumX = int(imageParams['SizeX'])
            self.maximCamera.NumY = int(imageParams['SizeY'])
            self.maximCamera.StartX = int(imageParams['OffX'])
            self.maximCamera.StartY = int(imageParams['OffY'])
            if imageParams['Speed'] == 'HiSpeed':
                self.maximCamera.FastReadout = True
            else:
                self.maximCamera.FastReadout = False
            suc = self.maximCamera.Expose(imageParams['Exposure'], 1)
            if not suc:
                self.logger.warning('Imaging no start, message: {0}'.format(mes))
                self.main.cameraStatusText.emit('ERROR')
                imageParams['Imagepath'] = ''
                self.mutexCancel.lock()
                self.cancel = True
                self.mutexCancel.unlock()
                # wait for start integrating
                while not self.cancel:
                    status = self.maximCamera.CameraStatus
                    if status not in [2]:
                        break
                    time.sleep(0.1)
        except Exception as e:
            self.logger.error('Could not start imaging, error: {0}'.format(e))
        finally:
            pass

        # loop for integrating
        self.main.cameraStatusText.emit('INTEGRATE')
        # wait for start integrating
        while not self.cancel:
            status = self.maximCamera.CameraStatus
            if status in [4, 5, 6, 8]:
                break
            time.sleep(0.1)

        # Loop for downloading
        self.main.imageIntegrated.emit()
        self.main.cameraStatusText.emit('DOWNLOAD')
        while not self.cancel:
            status = self.maximCamera.CameraStatus
            if status in [2]:
                break
            time.sleep(0.1)

        # Loop for saving
        self.main.imageDownloaded.emit()
        self.main.cameraStatusText.emit('SAVING')
        while not self.cancel:
            imageParams['Imagepath'] = imageParams['BaseDirImages'] + '/' + imageParams['File']
            self.maximCamera.SaveImage(imageParams['Imagepath'])
            # closes any open image view without notice
            self.maximApplication.CloseAll()
            break

        # finally idle
        self.main.cameraStatusText.emit('IDLE')
        self.main.cameraExposureTime.emit('')



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'hemisphere_window_ui.ui'
#
# Created by: PyQt5 UI code generator 5.10.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_HemisphereDialog(object):
    def setupUi(self, HemisphereDialog):
        HemisphereDialog.setObjectName("HemisphereDialog")
        HemisphereDialog.resize(791, 641)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(HemisphereDialog.sizePolicy().hasHeightForWidth())
        HemisphereDialog.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Arial")
        HemisphereDialog.setFont(font)
        self.hemisphere = QtWidgets.QWidget(HemisphereDialog)
        self.hemisphere.setEnabled(True)
        self.hemisphere.setGeometry(QtCore.QRect(10, 130, 771, 501))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hemisphere.sizePolicy().hasHeightForWidth())
        self.hemisphere.setSizePolicy(sizePolicy)
        self.hemisphere.setSizeIncrement(QtCore.QSize(0, 10))
        self.hemisphere.setAutoFillBackground(True)
        self.hemisphere.setStyleSheet("")
        self.hemisphere.setObjectName("hemisphere")
        self.btn_deletePoints = QtWidgets.QPushButton(HemisphereDialog)
        self.btn_deletePoints.setGeometry(QtCore.QRect(25, 15, 111, 31))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.btn_deletePoints.setFont(font)
        self.btn_deletePoints.setObjectName("btn_deletePoints")
        self.hemisphereBackground = QtWidgets.QLabel(HemisphereDialog)
        self.hemisphereBackground.setGeometry(QtCore.QRect(0, 0, 791, 126))
        self.hemisphereBackground.setText("")
        self.hemisphereBackground.setObjectName("hemisphereBackground")
        self.groupBox_2 = QtWidgets.QGroupBox(HemisphereDialog)
        self.groupBox_2.setGeometry(QtCore.QRect(235, 10, 166, 106))
        self.groupBox_2.setObjectName("groupBox_2")
        self.checkEditNone = QtWidgets.QRadioButton(self.groupBox_2)
        self.checkEditNone.setGeometry(QtCore.QRect(10, 20, 111, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEditNone.setFont(font)
        self.checkEditNone.setChecked(True)
        self.checkEditNone.setObjectName("checkEditNone")
        self.checkEditModelPoints = QtWidgets.QRadioButton(self.groupBox_2)
        self.checkEditModelPoints.setGeometry(QtCore.QRect(10, 40, 146, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEditModelPoints.setFont(font)
        self.checkEditModelPoints.setObjectName("checkEditModelPoints")
        self.checkEditHorizonMask = QtWidgets.QRadioButton(self.groupBox_2)
        self.checkEditHorizonMask.setGeometry(QtCore.QRect(10, 60, 141, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkEditHorizonMask.setFont(font)
        self.checkEditHorizonMask.setObjectName("checkEditHorizonMask")
        self.checkPolarAlignment = QtWidgets.QRadioButton(self.groupBox_2)
        self.checkPolarAlignment.setEnabled(True)
        self.checkPolarAlignment.setGeometry(QtCore.QRect(10, 80, 136, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkPolarAlignment.setFont(font)
        self.checkPolarAlignment.setObjectName("checkPolarAlignment")
        self.checkShowAlignmentStars = QtWidgets.QCheckBox(HemisphereDialog)
        self.checkShowAlignmentStars.setGeometry(QtCore.QRect(25, 50, 131, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowAlignmentStars.setFont(font)
        self.checkShowAlignmentStars.setObjectName("checkShowAlignmentStars")
        self.groupBox = QtWidgets.QGroupBox(HemisphereDialog)
        self.groupBox.setGeometry(QtCore.QRect(420, 10, 356, 106))
        self.groupBox.setObjectName("groupBox")
        self.label_100 = QtWidgets.QLabel(self.groupBox)
        self.label_100.setGeometry(QtCore.QRect(10, 40, 66, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_100.setFont(font)
        self.label_100.setObjectName("label_100")
        self.le_numberPointsImaged = QtWidgets.QLineEdit(self.groupBox)
        self.le_numberPointsImaged.setEnabled(False)
        self.le_numberPointsImaged.setGeometry(QtCore.QRect(70, 60, 26, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_numberPointsImaged.setFont(font)
        self.le_numberPointsImaged.setMouseTracking(False)
        self.le_numberPointsImaged.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_numberPointsImaged.setAcceptDrops(False)
        self.le_numberPointsImaged.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_numberPointsImaged.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_numberPointsImaged.setReadOnly(True)
        self.le_numberPointsImaged.setObjectName("le_numberPointsImaged")
        self.label_77 = QtWidgets.QLabel(self.groupBox)
        self.label_77.setGeometry(QtCore.QRect(10, 60, 66, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_77.setFont(font)
        self.label_77.setObjectName("label_77")
        self.bar_numberPointsImaged = QtWidgets.QProgressBar(self.groupBox)
        self.bar_numberPointsImaged.setGeometry(QtCore.QRect(105, 60, 241, 16))
        self.bar_numberPointsImaged.setMaximum(1000)
        self.bar_numberPointsImaged.setProperty("value", 1)
        self.bar_numberPointsImaged.setTextVisible(False)
        self.bar_numberPointsImaged.setObjectName("bar_numberPointsImaged")
        self.bar_numberPointsSlewed = QtWidgets.QProgressBar(self.groupBox)
        self.bar_numberPointsSlewed.setGeometry(QtCore.QRect(105, 40, 241, 16))
        self.bar_numberPointsSlewed.setMinimum(0)
        self.bar_numberPointsSlewed.setMaximum(1000)
        self.bar_numberPointsSlewed.setProperty("value", 0)
        self.bar_numberPointsSlewed.setTextVisible(False)
        self.bar_numberPointsSlewed.setOrientation(QtCore.Qt.Horizontal)
        self.bar_numberPointsSlewed.setInvertedAppearance(False)
        self.bar_numberPointsSlewed.setTextDirection(QtWidgets.QProgressBar.BottomToTop)
        self.bar_numberPointsSlewed.setObjectName("bar_numberPointsSlewed")
        self.le_numberPointsSlewed = QtWidgets.QLineEdit(self.groupBox)
        self.le_numberPointsSlewed.setEnabled(False)
        self.le_numberPointsSlewed.setGeometry(QtCore.QRect(70, 40, 26, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_numberPointsSlewed.setFont(font)
        self.le_numberPointsSlewed.setMouseTracking(False)
        self.le_numberPointsSlewed.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_numberPointsSlewed.setAcceptDrops(False)
        self.le_numberPointsSlewed.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_numberPointsSlewed.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_numberPointsSlewed.setReadOnly(True)
        self.le_numberPointsSlewed.setObjectName("le_numberPointsSlewed")
        self.le_numberPointsSolved = QtWidgets.QLineEdit(self.groupBox)
        self.le_numberPointsSolved.setEnabled(False)
        self.le_numberPointsSolved.setGeometry(QtCore.QRect(70, 80, 26, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_numberPointsSolved.setFont(font)
        self.le_numberPointsSolved.setMouseTracking(False)
        self.le_numberPointsSolved.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_numberPointsSolved.setAcceptDrops(False)
        self.le_numberPointsSolved.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_numberPointsSolved.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_numberPointsSolved.setReadOnly(True)
        self.le_numberPointsSolved.setObjectName("le_numberPointsSolved")
        self.bar_numberPointsSolved = QtWidgets.QProgressBar(self.groupBox)
        self.bar_numberPointsSolved.setGeometry(QtCore.QRect(105, 80, 241, 16))
        self.bar_numberPointsSolved.setMaximum(1000)
        self.bar_numberPointsSolved.setProperty("value", 1)
        self.bar_numberPointsSolved.setTextVisible(False)
        self.bar_numberPointsSolved.setObjectName("bar_numberPointsSolved")
        self.label_28 = QtWidgets.QLabel(self.groupBox)
        self.label_28.setGeometry(QtCore.QRect(10, 80, 66, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_28.setFont(font)
        self.label_28.setObjectName("label_28")
        self.le_numberPointsToProcess = QtWidgets.QLineEdit(self.groupBox)
        self.le_numberPointsToProcess.setEnabled(False)
        self.le_numberPointsToProcess.setGeometry(QtCore.QRect(125, 20, 26, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(8)
        font.setBold(False)
        font.setWeight(50)
        self.le_numberPointsToProcess.setFont(font)
        self.le_numberPointsToProcess.setMouseTracking(False)
        self.le_numberPointsToProcess.setFocusPolicy(QtCore.Qt.NoFocus)
        self.le_numberPointsToProcess.setAcceptDrops(False)
        self.le_numberPointsToProcess.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.le_numberPointsToProcess.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.le_numberPointsToProcess.setReadOnly(True)
        self.le_numberPointsToProcess.setObjectName("le_numberPointsToProcess")
        self.label_29 = QtWidgets.QLabel(self.groupBox)
        self.label_29.setGeometry(QtCore.QRect(10, 20, 121, 16))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_29.setFont(font)
        self.label_29.setObjectName("label_29")
        self.checkShowCelestial = QtWidgets.QCheckBox(HemisphereDialog)
        self.checkShowCelestial.setGeometry(QtCore.QRect(25, 70, 156, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowCelestial.setFont(font)
        self.checkShowCelestial.setObjectName("checkShowCelestial")
        self.checkShowMeridian = QtWidgets.QCheckBox(HemisphereDialog)
        self.checkShowMeridian.setGeometry(QtCore.QRect(25, 90, 156, 21))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.checkShowMeridian.setFont(font)
        self.checkShowMeridian.setObjectName("checkShowMeridian")
        self.hemisphereStar = QtWidgets.QWidget(HemisphereDialog)
        self.hemisphereStar.setEnabled(True)
        self.hemisphereStar.setGeometry(QtCore.QRect(10, 130, 771, 501))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hemisphereStar.sizePolicy().hasHeightForWidth())
        self.hemisphereStar.setSizePolicy(sizePolicy)
        self.hemisphereStar.setAutoFillBackground(False)
        self.hemisphereStar.setStyleSheet("")
        self.hemisphereStar.setObjectName("hemisphereStar")
        self.hemisphereMoving = QtWidgets.QWidget(HemisphereDialog)
        self.hemisphereMoving.setEnabled(True)
        self.hemisphereMoving.setGeometry(QtCore.QRect(10, 130, 771, 501))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hemisphereMoving.sizePolicy().hasHeightForWidth())
        self.hemisphereMoving.setSizePolicy(sizePolicy)
        self.hemisphereMoving.setAutoFillBackground(False)
        self.hemisphereMoving.setStyleSheet("")
        self.hemisphereMoving.setObjectName("hemisphereMoving")
        self.hemisphereBackground.raise_()
        self.btn_deletePoints.raise_()
        self.groupBox_2.raise_()
        self.hemisphere.raise_()
        self.checkShowAlignmentStars.raise_()
        self.groupBox.raise_()
        self.checkShowCelestial.raise_()
        self.checkShowMeridian.raise_()
        self.hemisphereStar.raise_()
        self.hemisphereMoving.raise_()

        self.retranslateUi(HemisphereDialog)
        QtCore.QMetaObject.connectSlotsByName(HemisphereDialog)

    def retranslateUi(self, HemisphereDialog):
        _translate = QtCore.QCoreApplication.translate
        HemisphereDialog.setWindowTitle(_translate("HemisphereDialog", "Hemisphere"))
        self.btn_deletePoints.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p>Delete all points on coordinate window including Base / Model</p></body></html>"))
        self.btn_deletePoints.setText(_translate("HemisphereDialog", "Clear Points"))
        self.hemisphereBackground.setProperty("color", _translate("HemisphereDialog", "blue"))
        self.groupBox_2.setTitle(_translate("HemisphereDialog", "Operation Mode"))
        self.checkEditNone.setText(_translate("HemisphereDialog", "Normal mode"))
        self.checkEditModelPoints.setText(_translate("HemisphereDialog", "Edit Model Points"))
        self.checkEditHorizonMask.setText(_translate("HemisphereDialog", "Edit Horizon Mask"))
        self.checkPolarAlignment.setText(_translate("HemisphereDialog", "Select PolarAlign"))
        self.checkShowAlignmentStars.setText(_translate("HemisphereDialog", "Show align stars"))
        self.groupBox.setTitle(_translate("HemisphereDialog", "Mode Build Progress"))
        self.label_100.setText(_translate("HemisphereDialog", "Slewed:"))
        self.le_numberPointsImaged.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_numberPointsImaged.setText(_translate("HemisphereDialog", "0"))
        self.label_77.setText(_translate("HemisphereDialog", "Imaged:"))
        self.bar_numberPointsSlewed.setFormat(_translate("HemisphereDialog", "%p%"))
        self.le_numberPointsSlewed.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_numberPointsSlewed.setText(_translate("HemisphereDialog", "0"))
        self.le_numberPointsSolved.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_numberPointsSolved.setText(_translate("HemisphereDialog", "0"))
        self.label_28.setText(_translate("HemisphereDialog", "Solved:"))
        self.le_numberPointsToProcess.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p><span style=\" font-size:10pt;\">Progress in modeling.</span></p></body></html>"))
        self.le_numberPointsToProcess.setText(_translate("HemisphereDialog", "0"))
        self.label_29.setText(_translate("HemisphereDialog", "Points to process:"))
        self.checkShowCelestial.setText(_translate("HemisphereDialog", "Show celestial paths"))
        self.checkShowMeridian.setText(_translate("HemisphereDialog", "Show meridian limits"))




############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.5
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import PyQt5
import time
import indi.indi_xml as indiXML


class INDICamera:
    logger = logging.getLogger(__name__)

    # timeout for getting an download is 30 seconds
    MAX_DOWNLOAD_TIMEOUT = 30
    START_CAMERA_TIMEOUT = 3

    def __init__(self, main, app, data):
        # make main sources available
        self.main = main
        self.app = app
        self.data = data
        self.cancel = False
        self.mutexCancel = PyQt5.QtCore.QMutex()
        self.mutexReceived = PyQt5.QtCore.QMutex()

        self.application = dict()
        self.application['Available'] = False
        self.application['Name'] = ''
        self.application['InstallPath'] = ''
        self.application['Status'] = ''
        self.application['Runtime'] = 'Sequence Generator.exe'

        self.counter = 0
        self.receivedImage = True

        self.application['Status'] = ''
        self.application['CONNECTION'] = {'CONNECT': 'Off'}
        self.application['Available'] = True
        self.application['Name'] = 'INDICamera'
        self.application['InstallPath'] = ''

        self.app.workerINDI.receivedImage.connect(self.setReceivedImage)

    def start(self):
        # connect the camera if not present
        timeStart = time.time()
        while True:
            if time.time() - timeStart > self.START_CAMERA_TIMEOUT:
                self.app.messageQueue.put('Timeout connect camera\n')
                break
            if self.app.workerINDI.cameraDevice:
                if 'CONNECTION' in self.app.workerINDI.data['Device'][self.app.workerINDI.cameraDevice]:
                    # Enable BLOB mode it also enables listen to send images
                    self.app.INDICommandQueue.put(indiXML.enableBLOB('Also', indi_attr={'device': self.app.workerINDI.cameraDevice}))
                    break
            time.sleep(0.1)
        self.connect()

    def stop(self):
        pass

    def setReceivedImage(self, status):
        self.mutexReceived.lock()
        if status:
            self.receivedImage = True
        else:
            self.receivedImage = False
        self.mutexReceived.unlock()

    def getStatus(self):
        # check if INDIClient is running and camera device is there
        if self.app.workerINDI.isRunning:
            self.application['Available'] = True
            if self.app.workerINDI.cameraDevice != '':
                self.application['Status'] = 'OK'
                self.application['Name'] = self.app.workerINDI.cameraDevice
                # check if data from INDI server already received
                if 'CONNECTION' in self.app.workerINDI.data['Device'][self.app.workerINDI.cameraDevice]:
                    self.data['CONNECTION']['CONNECT'] = self.app.workerINDI.data['Device'][self.app.workerINDI.cameraDevice]['CONNECTION']['CONNECT']
                else:
                    self.logger.error('Unknown camera status')
            else:
                self.application['Status'] = 'ERROR'
        else:
            self.application['Available'] = False

    def getCameraProps(self):
        self.data['Gain'] = 'High'
        self.data['Speed'] = 'High'
        self.data['CCD_INFO'] = {}
        self.data['CCD_INFO']['CCD_MAX_X'] = self.app.workerINDI.data['Device'][self.app.workerINDI.cameraDevice]['CCD_INFO']['CCD_MAX_X']
        self.data['CCD_INFO']['CCD_MAX_Y'] = self.app.workerINDI.data['Device'][self.app.workerINDI.cameraDevice]['CCD_INFO']['CCD_MAX_Y']

    def getImage(self, imageParams):
        if self.application['Status'] != 'OK':
            return
        self.data['Imaging'] = True
        self.mutexCancel.lock()
        self.cancel = False
        self.mutexCancel.unlock()

        binning = int(imageParams['Binning'])
        exposure = int(imageParams['Exposure'])
        filename = imageParams['File']
        path = imageParams['BaseDirImages']
        imagePath = path + '/' + filename
        # setting image path in INDI client to know where to store the image
        self.app.workerINDI.imagePath = imagePath

        cam = self.app.workerINDI.data['Device'][self.app.workerINDI.cameraDevice]
        if self.app.workerINDI.cameraDevice != '' and cam['CONNECTION']['CONNECT'] == 'On':
            # Enable BLOB mode.
            self.app.INDICommandQueue.put(indiXML.enableBLOB('Also', indi_attr={'device': self.app.workerINDI.cameraDevice}))
            # set to raw - no compression mode
            self.app.INDICommandQueue.put(
                indiXML.newSwitchVector([indiXML.oneSwitch('On', indi_attr={'name': 'CCD_COMPRESS'})],
                                        indi_attr={'name': 'CCD_COMPRESSION', 'device': self.app.workerINDI.cameraDevice}))
            # set frame type
            self.app.INDICommandQueue.put(
                indiXML.newSwitchVector([indiXML.oneSwitch('On', indi_attr={'name': 'FRAME_LIGHT'})],
                                        indi_attr={'name': 'CCD_FRAME_TYPE', 'device': self.app.workerINDI.cameraDevice}))
            # set binning
            self.app.INDICommandQueue.put(
                indiXML.newNumberVector([indiXML.oneNumber(binning, indi_attr={'name': 'HOR_BIN'}),
                                         indiXML.oneNumber(binning, indi_attr={'name': 'VER_BIN'})],
                                        indi_attr={'name': 'CCD_BINNING', 'device': self.app.workerINDI.cameraDevice}))
            # set subframe
            self.app.INDICommandQueue.put(
                indiXML.newNumberVector([indiXML.oneNumber(imageParams['OffX'], indi_attr={'name': 'X'}),
                                         indiXML.oneNumber(imageParams['OffY'], indi_attr={'name': 'Y'}),
                                         indiXML.oneNumber(imageParams['SizeX'], indi_attr={'name': 'WIDTH'}),
                                         indiXML.oneNumber(imageParams['SizeY'], indi_attr={'name': 'HEIGHT'})],
                                        indi_attr={'name': 'CCD_FRAME', 'device': self.app.workerINDI.cameraDevice}))
            # Request image.
            self.app.INDICommandQueue.put(
                indiXML.newNumberVector([indiXML.oneNumber(exposure, indi_attr={'name': 'CCD_EXPOSURE_VALUE'})],
                                        indi_attr={'name': 'CCD_EXPOSURE', 'device': self.app.workerINDI.cameraDevice}))
        else:
            self.mutexCancel.lock()
            self.cancel = True
            self.mutexCancel.unlock()

        self.mutexReceived.lock()
        self.receivedImage = False
        self.mutexReceived.unlock()

        # waiting for start integrating
        self.main.cameraStatusText.emit('START')
        while not self.cancel:
            if 'CONNECTION' and 'CCD_EXPOSURE' in cam:
                if cam['CONNECTION']['CONNECT'] == 'On':
                    if cam['CCD_EXPOSURE']['state'] in ['Busy']:
                        break
                else:
                    self.main.cameraStatusText.emit('DISCONN')
            else:
                self.main.cameraStatusText.emit('ERROR')
            time.sleep(0.1)

        # loop for integrating
        self.main.cameraStatusText.emit('INTEGRATE')
        while not self.cancel:
            if 'CONNECTION' and 'CCD_EXPOSURE' in cam:
                if cam['CONNECTION']['CONNECT'] == 'On':
                    if not float(cam['CCD_EXPOSURE']['CCD_EXPOSURE_VALUE']):
                        break
                else:
                    self.main.cameraStatusText.emit('DISCONN')
            else:
                self.main.cameraStatusText.emit('ERROR')
            if 'CCD_EXPOSURE' in cam:
                self.main.cameraExposureTime.emit('{0:02.0f}'.format(float(cam['CCD_EXPOSURE']['CCD_EXPOSURE_VALUE'])))
            else:
                self.main.cameraExposureTime.emit('')
            time.sleep(0.1)

        # loop for download
        self.main.imageIntegrated.emit()
        self.main.cameraStatusText.emit('DOWNLOAD')
        while not self.cancel:
            if 'CCD_EXPOSURE' in cam:
                if cam['CONNECTION']['CONNECT'] == 'On':
                    if cam['CCD_EXPOSURE']['state'] in ['Ok', 'Idle']:
                        break
                    elif cam['CCD_EXPOSURE']['state'] == 'Error':
                        self.main.cameraStatusText.emit('ERROR')
                else:
                    self.main.cameraStatusText.emit('DISCONN')
            else:
                self.main.cameraStatusText.emit('ERROR')
            if 'CCD_EXPOSURE' in cam:
                self.main.cameraExposureTime.emit('{0:02.0f}'.format(float(cam['CCD_EXPOSURE']['CCD_EXPOSURE_VALUE'])))
            else:
                self.main.cameraExposureTime.emit('')
            time.sleep(0.1)

        # loop for saving
        self.main.imageDownloaded.emit()
        self.main.cameraStatusText.emit('SAVING')
        while not self.cancel:
            if self.receivedImage:
                break
            time.sleep(0.1)

        # finally idle
        self.main.cameraStatusText.emit('IDLE')
        self.main.cameraExposureTime.emit('')
        imageParams['Imagepath'] = self.app.workerINDI.imagePath
        self.app.workerINDI.imagePath = ''

    def connect(self):
        # connect the camera
        if self.app.workerINDI.cameraDevice != '':
            if self.app.workerINDI.data['Device'][self.app.workerINDI.cameraDevice]['CONNECTION']['CONNECT'] == 'Off':
                self.app.INDICommandQueue.put(indiXML.newSwitchVector([indiXML.oneSwitch('On', indi_attr={'name': 'CONNECT'})], indi_attr={'name': 'CONNECTION', 'device': self.app.workerINDI.cameraDevice}))

    def disconnect(self):
        if self.app.workerINDI.cameraDevice != '':
            if self.app.workerINDI.data['Device'][self.app.workerINDI.cameraDevice]['CONNECTION']['CONNECT'] == 'On':
                self.app.INDICommandQueue.put(indiXML.newSwitchVector([indiXML.oneSwitch('Off', indi_attr={'name': 'CONNECT'})], indi_attr={'name': 'CONNECTION', 'device': self.app.workerINDI.cameraDevice}))



############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################
import PyQt5
import platform
import logging
from win32com.client.dynamic import Dispatch
import pythoncom


class PinPoint:
    logger = logging.getLogger(__name__)

    def __init__(self, main, app, data):
        self.main = main
        self.app = app
        self.data = data
        self.cancel = False
        self.mutexCancel = PyQt5.QtCore.QMutex()

        self.application = dict()
        self.application['Available'] = False
        self.application['Name'] = ''
        self.application['InstallPath'] = ''
        self.application['Status'] = ''
        self.application['Runtime'] = 'PinPoint.dll'
        self.pinpoint = None

        if platform.system() == 'Windows':
            # sgpro only supported on local machine
            self.application['Available'], self.application['Name'], self.application['InstallPath'] = self.app.checkRegistrationKeys('PinPoint')
            if self.application['Available']:
                self.app.messageQueue.put('Found Astrometry: {0}\n'.format(self.application['Name']))
                self.logger.info('Name: {0}, Path: {1}'.format(self.application['Name'], self.application['InstallPath']))
            else:
                self.logger.info('Application PinPoint not found on computer')

        # slots:
        self.app.ui.btn_selectCatalogue.clicked.connect(self.setCatalogue)

    def setCatalogue(self):
        value = self.app.selectDir(self.app, 'Set PinPoint Star Catalogue', '/')
        if value != '':
            self.app.ui.le_pinpointCatalogue.setText(value)
        else:
            self.logger.warning('No catalogue file selected')

    def start(self):
        pythoncom.CoInitialize()
        try:
            if self.app.ui.le_pinpointCatalogue.text() != '':
                if 'UCAC4' in self.app.ui.le_pinpointCatalogue.text():
                    cat = 11
                elif 'GSC' in self.app.ui.le_pinpointCatalogue.text():
                    cat = 3
                elif 'UCAC3' in self.app.ui.le_pinpointCatalogue.text():
                    cat = 10
                elif 'USNO' in self.app.ui.le_pinpointCatalogue.text():
                    cat = 7
                else:
                    self.logger.info('Pinpoint catalogue could not be configured')
                    self.application['Status'] = 'ERROR'
                    self.app.messageQueue.put('#BRPinpoint catalogue could not be configured\n')
                    return
            else:
                self.logger.info('Pinpoint catalogue not defined')
                self.application['Status'] = 'ERROR'
                self.app.messageQueue.put('#BRPinpoint catalogue not defined !\n')
                return
            self.pinpoint = Dispatch('PinPoint.Plate')
            self.pinpoint.Catalog = cat
            self.pinpoint.CatalogPath = self.app.ui.le_pinpointCatalogue.text()
        except Exception as e:
            self.logger.info('Pinpoint could not be started, error:{0}'.format(e))
            self.application['Status'] = 'ERROR'
        finally:
            self.app.messageQueue.put('Catalogue path: {0}, number scheme PinPoint: {1}\n'.format(self.app.ui.le_pinpointCatalogue.text(), cat))
            pass

    def stop(self):
        try:
            if self.pinpoint:
                self.data['CONNECTION']['CONNECT'] = 'Off'
        except Exception as e:
            self.logger.error('Could not stop maxim, error: {0}'.format(e))
            self.application['Status'] = 'ERROR'
        finally:
            self.data['CONNECTION']['CONNECT'] = 'Off'
            self.pinpoint = None
            pythoncom.CoUninitialize()

    def getStatus(self):
        if self.pinpoint:
            self.data['CONNECTION']['CONNECT'] = 'On'
            self.application['Status'] = 'OK'
        else:
            self.application['Status'] = 'ERROR'
            self.data['CONNECTION']['CONNECT'] = 'Off'

    def solveImage(self, imageParams):

        try:
            # waiting for start solving
            self.main.astrometryStatusText.emit('START')
            if not self.pinpoint.AttachFITS(imageParams['Imagepath']):
                return
            self.pinpoint.ArcsecPerPixelHoriz = imageParams['ScaleHint']
            self.pinpoint.ArcsecPerPixelVert = imageParams['ScaleHint']
            self.pinpoint.RightAscension = self.pinpoint.TargetRightAscension
            self.pinpoint.Declination = self.pinpoint.TargetDeclination

            # loop for solve
            self.main.astrometryStatusText.emit('SOLVE')
            try:
                self.pinpoint.Solve()
                imageParams['Solved'] = True
            except pythoncom.com_error as e:
                imageParams['Solved'] = False
                imageParams['Message'] = e.excepinfo[2][:80]
            finally:
                pass
            # loop for get data
            self.main.astrometryStatusText.emit('GET DATA')
            self.pinpoint.DetachFITS()
            if imageParams['Solved']:
                imageParams['DecJ2000Solved'] = float(self.pinpoint.Declination)
                imageParams['RaJ2000Solved'] = float(self.pinpoint.RightAscension)
                imageParams['Scale'] = float(self.pinpoint.ArcsecPerPixelHoriz)
                imageParams['Angle'] = float(self.pinpoint.RollAngle)
                imageParams['TimeTS'] = 2.0
                imageParams['Message'] = 'OK'
            else:
                imageParams['DecJ2000Solved'] = 0
                imageParams['RaJ2000Solved'] = 0
                imageParams['Scale'] = 0
                imageParams['Angle'] = 0
                imageParams['TimeTS'] = 2.0

        except Exception as e:
            imageParams['Solved'] = False
            imageParams['Message'] = 'Solve failed'
            self.logger.error('PinPoint solving error -> error: {0}'.format(e))
        finally:
            pass

        # finally idle
        self.main.imageDataDownloaded.emit()
        self.main.astrometryStatusText.emit('IDLE')
        self.main.astrometrySolvingTime.emit('')
